#define PERL_constant_NOTFOUND	1
#define PERL_constant_NOTDEF	2
#define PERL_constant_ISIV	3
#define PERL_constant_ISNO	4
#define PERL_constant_ISNV	5
#define PERL_constant_ISPV	6
#define PERL_constant_ISPVN	7
#define PERL_constant_ISSV	8
#define PERL_constant_ISUNDEF	9
#define PERL_constant_ISUV	10
#define PERL_constant_ISYES	11

#ifndef NVTYPE
typedef double NV; /* 5.6 and later define NVTYPE, and typedef NV to it.  */
#endif
#ifndef aTHX_
#define aTHX_ /* 5.6 or later define this for threading support.  */
#endif
#ifndef pTHX_
#define pTHX_ /* 5.6 or later define this for threading support.  */
#endif

/* This allows slightly more efficient code on !USE_ITHREADS: */
#ifdef USE_ITHREADS
#  define COP_FILE(c)	CopFILE(c)
#  define COP_FILE_F	"s"
#else
#  define COP_FILE(c)	CopFILESV(c)
#  define COP_FILE_F	SVf
#endif

static void
constant_add_symbol(pTHX_  HV *hash, const char *name, I32 namelen, SV *value) {
    HE *he = (HE*) hv_common_key_len(hash, name, namelen, HV_FETCH_LVALUE, NULL,
				     0);
    SV *sv;

    if (!he) {
        croak("Couldn't add key '%s' to %%POSIX::2008::",
		   name);
    }
    sv = HeVAL(he);
    if (SvOK(sv) || SvTYPE(sv) == SVt_PVGV) {
	/* Someone has been here before us - have to make a real sub.  */
	newCONSTSUB(hash, name, value);
    } else {
	SvUPGRADE(sv, SVt_RV);
	SvRV_set(sv, value);
	SvROK_on(sv);
	SvREADONLY_on(value);
    }
}



#ifndef SYMBIAN

/* Store a hash of all symbols missing from the package. To avoid trampling on
   the package namespace (uninvited) put each package's hash in our namespace.
   To avoid creating lots of typeblogs and symbol tables for sub-packages, put
   each package's hash into one hash in our namespace.  */

static HV *
get_missing_hash(pTHX) {
    HV *const parent
	= get_hv("ExtUtils::Constant::ProxySubs::Missing", GVf_MULTI);
    /* We could make a hash of hashes directly, but this would confuse anything
	at Perl space that looks at us, and as we're visible in Perl space,
	best to play nice. */
    SV *const *const ref
	= hv_fetch(parent, "POSIX::2008", 11, TRUE);
    HV *new_hv;

    if (!ref)
	return NULL;

    if (SvROK(*ref))
	return (HV*) SvRV(*ref);

    new_hv = newHV();
    SvUPGRADE(*ref, SVt_RV);
    SvRV_set(*ref, (SV *)new_hv);
    SvROK_on(*ref);
    return new_hv;
}

#endif

struct notfound_s {const char *name; I32 namelen;} ;

    static const struct notfound_s values_for_notfound[] =
      {
#ifndef AT_EACCESS
        { "AT_EACCESS", 10 },
#endif
#ifndef AT_EMPTY_PATH
        { "AT_EMPTY_PATH", 13 },
#endif
#ifndef AT_FDCWD
        { "AT_FDCWD", 8 },
#endif
#ifndef AT_NO_AUTOMOUNT
        { "AT_NO_AUTOMOUNT", 15 },
#endif
#ifndef AT_REMOVEDIR
        { "AT_REMOVEDIR", 12 },
#endif
#ifndef AT_SYMLINK_FOLLOW
        { "AT_SYMLINK_FOLLOW", 17 },
#endif
#ifndef AT_SYMLINK_NOFOLLOW
        { "AT_SYMLINK_NOFOLLOW", 19 },
#endif
#ifndef BOOT_TIME
        { "BOOT_TIME", 9 },
#endif
#ifndef CLOCK_BOOTTIME
        { "CLOCK_BOOTTIME", 14 },
#endif
#ifndef CLOCK_HIGHRES
        { "CLOCK_HIGHRES", 13 },
#endif
#ifndef CLOCK_MONOTONIC
        { "CLOCK_MONOTONIC", 15 },
#endif
#ifndef CLOCK_MONOTONIC_COARSE
        { "CLOCK_MONOTONIC_COARSE", 22 },
#endif
#ifndef CLOCK_MONOTONIC_FAST
        { "CLOCK_MONOTONIC_FAST", 20 },
#endif
#ifndef CLOCK_MONOTONIC_PRECISE
        { "CLOCK_MONOTONIC_PRECISE", 23 },
#endif
#ifndef CLOCK_MONOTONIC_RAW
        { "CLOCK_MONOTONIC_RAW", 19 },
#endif
#ifndef CLOCK_PROCESS_CPUTIME_ID
        { "CLOCK_PROCESS_CPUTIME_ID", 24 },
#endif
#ifndef CLOCK_REALTIME
        { "CLOCK_REALTIME", 14 },
#endif
#ifndef CLOCK_REALTIME_COARSE
        { "CLOCK_REALTIME_COARSE", 21 },
#endif
#ifndef CLOCK_REALTIME_FAST
        { "CLOCK_REALTIME_FAST", 19 },
#endif
#ifndef CLOCK_REALTIME_PRECISE
        { "CLOCK_REALTIME_PRECISE", 22 },
#endif
#ifndef CLOCK_SOFTTIME
        { "CLOCK_SOFTTIME", 14 },
#endif
#ifndef CLOCK_THREAD_CPUTIME_ID
        { "CLOCK_THREAD_CPUTIME_ID", 23 },
#endif
#ifndef CLOCK_UPTIME
        { "CLOCK_UPTIME", 12 },
#endif
#ifndef CLOCK_UPTIME_FAST
        { "CLOCK_UPTIME_FAST", 17 },
#endif
#ifndef CLOCK_UPTIME_PRECISE
        { "CLOCK_UPTIME_PRECISE", 20 },
#endif
#ifndef _CS_GNU_LIBC_VERSION
        { "_CS_GNU_LIBC_VERSION", 20 },
#endif
#ifndef _CS_GNU_LIBPTHREAD_VERSION
        { "_CS_GNU_LIBPTHREAD_VERSION", 26 },
#endif
#ifndef _CS_PATH
        { "_CS_PATH", 8 },
#endif
#ifndef DEAD_PROCESS
        { "DEAD_PROCESS", 12 },
#endif
#ifndef F_DUPFD
        { "F_DUPFD", 7 },
#endif
#ifndef F_DUPFD_CLOEXEC
        { "F_DUPFD_CLOEXEC", 15 },
#endif
#ifndef F_GETFD
        { "F_GETFD", 7 },
#endif
#ifndef F_SETFD
        { "F_SETFD", 7 },
#endif
#ifndef F_GETFL
        { "F_GETFL", 7 },
#endif
#ifndef F_SETFL
        { "F_SETFL", 7 },
#endif
#ifndef F_GETLK
        { "F_GETLK", 7 },
#endif
#ifndef F_SETLK
        { "F_SETLK", 7 },
#endif
#ifndef F_SETLKW
        { "F_SETLKW", 8 },
#endif
#ifndef F_GETOWN
        { "F_GETOWN", 8 },
#endif
#ifndef F_SETOWN
        { "F_SETOWN", 8 },
#endif
#ifndef F_RDLCK
        { "F_RDLCK", 7 },
#endif
#ifndef F_UNLCK
        { "F_UNLCK", 7 },
#endif
#ifndef F_WRLCK
        { "F_WRLCK", 7 },
#endif
#ifndef FD_CLOEXEC
        { "FD_CLOEXEC", 10 },
#endif
#ifndef FNM_CASEFOLD
        { "FNM_CASEFOLD", 12 },
#endif
#ifndef FNM_FILE_NAME
        { "FNM_FILE_NAME", 13 },
#endif
#ifndef FNM_LEADING_DIR
        { "FNM_LEADING_DIR", 15 },
#endif
#ifndef FNM_NOESCAPE
        { "FNM_NOESCAPE", 12 },
#endif
#ifndef FNM_NOMATCH
        { "FNM_NOMATCH", 11 },
#endif
#ifndef FNM_PATHNAME
        { "FNM_PATHNAME", 12 },
#endif
#ifndef FNM_PERIOD
        { "FNM_PERIOD", 10 },
#endif
#ifndef FP_INFINITE
        { "FP_INFINITE", 11 },
#endif
#ifndef FP_NAN
        { "FP_NAN", 6 },
#endif
#ifndef FP_NORMAL
        { "FP_NORMAL", 9 },
#endif
#ifndef FP_SUBNORMAL
        { "FP_SUBNORMAL", 12 },
#endif
#ifndef FP_ZERO
        { "FP_ZERO", 7 },
#endif
#ifndef INIT_PROCESS
        { "INIT_PROCESS", 12 },
#endif
#ifndef ITIMER_PROF
        { "ITIMER_PROF", 11 },
#endif
#ifndef ITIMER_REAL
        { "ITIMER_REAL", 11 },
#endif
#ifndef ITIMER_VIRTUAL
        { "ITIMER_VIRTUAL", 14 },
#endif
#ifndef LOGIN_PROCESS
        { "LOGIN_PROCESS", 13 },
#endif
#ifndef NEW_TIME
        { "NEW_TIME", 8 },
#endif
#ifndef O_ACCMODE
        { "O_ACCMODE", 9 },
#endif
#ifndef O_APPEND
        { "O_APPEND", 8 },
#endif
#ifndef O_CLOEXEC
        { "O_CLOEXEC", 9 },
#endif
#ifndef O_CREAT
        { "O_CREAT", 7 },
#endif
#ifndef O_DIRECTORY
        { "O_DIRECTORY", 11 },
#endif
#ifndef O_DSYNC
        { "O_DSYNC", 7 },
#endif
#ifndef O_EXEC
        { "O_EXEC", 6 },
#endif
#ifndef O_NOCTTY
        { "O_NOCTTY", 8 },
#endif
#ifndef O_NOFOLLOW
        { "O_NOFOLLOW", 10 },
#endif
#ifndef O_NONBLOCK
        { "O_NONBLOCK", 10 },
#endif
#ifndef O_RDONLY
        { "O_RDONLY", 8 },
#endif
#ifndef O_RDWR
        { "O_RDWR", 6 },
#endif
#ifndef O_RSYNC
        { "O_RSYNC", 7 },
#endif
#ifndef O_SEARCH
        { "O_SEARCH", 8 },
#endif
#ifndef O_SYNC
        { "O_SYNC", 6 },
#endif
#ifndef O_TMPFILE
        { "O_TMPFILE", 9 },
#endif
#ifndef O_TRUNC
        { "O_TRUNC", 7 },
#endif
#ifndef O_TTY_INIT
        { "O_TTY_INIT", 10 },
#endif
#ifndef O_WRONLY
        { "O_WRONLY", 8 },
#endif
#ifndef OLD_TIME
        { "OLD_TIME", 8 },
#endif
#ifndef POSIX_FADV_NORMAL
        { "POSIX_FADV_NORMAL", 17 },
#endif
#ifndef POSIX_FADV_SEQUENTIAL
        { "POSIX_FADV_SEQUENTIAL", 21 },
#endif
#ifndef POSIX_FADV_RANDOM
        { "POSIX_FADV_RANDOM", 17 },
#endif
#ifndef POSIX_FADV_NOREUSE
        { "POSIX_FADV_NOREUSE", 18 },
#endif
#ifndef POSIX_FADV_WILLNEED
        { "POSIX_FADV_WILLNEED", 19 },
#endif
#ifndef POSIX_FADV_DONTNEED
        { "POSIX_FADV_DONTNEED", 19 },
#endif
#ifndef RESOLVE_BENEATH
        { "RESOLVE_BENEATH", 15 },
#endif
#ifndef RESOLVE_IN_ROOT
        { "RESOLVE_IN_ROOT", 15 },
#endif
#ifndef RESOLVE_NO_MAGICLINKS
        { "RESOLVE_NO_MAGICLINKS", 21 },
#endif
#ifndef RESOLVE_NO_SYMLINKS
        { "RESOLVE_NO_SYMLINKS", 19 },
#endif
#ifndef RESOLVE_NO_XDEV
        { "RESOLVE_NO_XDEV", 15 },
#endif
#ifndef RESOLVE_CACHED
        { "RESOLVE_CACHED", 14 },
#endif
#ifndef RTLD_GLOBAL
        { "RTLD_GLOBAL", 11 },
#endif
#ifndef RTLD_LAZY
        { "RTLD_LAZY", 9 },
#endif
#ifndef RTLD_LOCAL
        { "RTLD_LOCAL", 10 },
#endif
#ifndef RTLD_NOW
        { "RTLD_NOW", 8 },
#endif
#ifndef RUN_LVL
        { "RUN_LVL", 7 },
#endif
#ifndef SEEK_SET
        { "SEEK_SET", 8 },
#endif
#ifndef SEEK_CUR
        { "SEEK_CUR", 8 },
#endif
#ifndef SEEK_END
        { "SEEK_END", 8 },
#endif
#ifndef TIMER_ABSTIME
        { "TIMER_ABSTIME", 13 },
#endif
#ifndef USER_PROCESS
        { "USER_PROCESS", 12 },
#endif
#ifndef UTIME_NOW
        { "UTIME_NOW", 9 },
#endif
#ifndef UTIME_OMIT
        { "UTIME_OMIT", 10 },
#endif
        { NULL, 0 } };
struct iv_s {const char *name; I32 namelen; IV value;};
